/** Wertet einen Infix-String aus. Es sind die Zeichen '(0123456789,.+*-/)' inklusive
  * der Klammern erlaubt. Der String muss mit einem fremden Zeichen beendet
  * werden, damit er korrekt berechnet wird.
  */
public class InfixCalculation{
  private static String rechnung;
  private static int pos;
  private static char zeichen;

  private static void weiter(){zeichen=rechnung.charAt(pos++);}
  private static boolean ziffernzeichen(){return zeichen>='0' && zeichen<='9';}

/** Eine Zahl besteht aus den Ziffern '0' bis '9'. Es darf ein '.' oder ',' folgen mit
  * nachgestelltem Nachkommateil, der wieder aus den obigen Ziffern bestehen kann.
  */
  private static double zahl(){
    double a=0, b=0, d=10;
    while(ziffernzeichen()) {a=a*10+(zeichen-'0'); weiter();}
    if(zeichen==',' || zeichen=='.'){
      weiter();
      while(ziffernzeichen()){
        b+=(zeichen-'0')/d;
        d*=10;
        weiter();
      }
    }
    return a+b;
  }

/** Wertet einen Term aus. Die Production lautet
  * TERM:
  *     ZAHL
  *     (SUMME)
  *     leer
  * TERM hat den tiefsten Level in der rekursiven Hierarchie und wird verwendet
  * wie eine Zahl, nur dass diese "Zahl" eben auch wieder eine ganze SUMME sein kann.
  * Mathematisch exakt darf ein Term natrlich nicht leer sein, dieser Fall wurde aber
  * aus Grnden der besseren Fehlertoleranz mit aufgenommen. Die Funktion term gibt
  * dann den Wert 1 zurck.
  */
  private static double term(){
    if(ziffernzeichen()) return zahl();
    if(zeichen=='('){
      weiter();
      double temp=summe();
      if(zeichen==')') weiter();
      return temp;
    }
    return 1;
  }

/** Wertet ein Produkt aus. Die Production lautet
  * PRODUKT:
  *        TERM
  *        PRODUKT*TERM
  *        PRODUKT/TERM
  * PRODUKT liegt in der rekursiven Hierarchie einen Level unter SUMME und kann deshalb
  * dafr sorgen, dass die 'Punkt vor Strich'-Regel eingehalten wird, indem es alle Punktrechnungen
  * gesammelt ausfhrt.
  */
  private static double produkt(){
    double produkt=term();
    while(zeichen=='*' || zeichen=='/'){
      char multop=zeichen;
      weiter();
      if(multop=='*') produkt*=term(); else produkt/=term();
    }
    return produkt;
  }

/** Wertet eine Summe aus. Die Production lautet
  * SUMME:
  *      PRODUKT
  *      +PRODUKT
  *      -PRODUKT
  *      SUMME+PRODUKT
  *      SUMME-PRODUKT
  * SUMME hat den hchsten Level in der rekursiven Hierarchie und wird deshalb
  * als erstes aufgerufen.
  */
  private static double summe(){
    double summe=0;
    if(zeichen!='+' && zeichen!='-') summe=produkt();
    while(zeichen=='+' || zeichen=='-'){
      char addop=zeichen;
      weiter();
      if(addop=='+') summe+=produkt(); else summe-=produkt();
    }
    return summe;
  }

/** Wertet einen gewhnlichen Infix-Ausdruck mit den vier Grundrechenarten und Klammern
  * aus. Der String muss mit einem fremden Zeichen enden, z.B. '='.
  */
  public static double ergebnis(String s){
    rechnung=s;
    pos=0;
    weiter();
    return summe();
  }

  public static void main(String[] args) throws Exception{
    String aufg;
    do{
      System.out.println("Geben Sie eine Rechnung ein und beenden Sie sie mit =");
      aufg=LineInput.readString();
      if(!aufg.equals("")) System.out.println("Ergebnis: "+ergebnis(aufg));
    } while(!aufg.equals(""));
  }
}
