/* Diese Klasse ist die Lsung einer Aufgabe aus der Informatik-
 * Olympiade. Es verwendet die altmodischen GUI-Elemente der
 * ersten Generation, die nicht auf jedem Betriebssystem gleich
 * aussehen, aber dennoch auf allen funktionieren.
 * Anfangs zieht man mit der Maus einige Striche. Die relativen
 * Positionen und relativen Lngen, die diese zueinander haben,
 * haben danach auch die Striche der nchsten Generation zur
 * aktuellen.
*/
import java.awt.*;
import java.awt.event.*;
import java.util.*;

class Strich{
  private float xA, yA, xB, yB;

  public Strich(float xA, float yA, float xB, float yB){
    this.xA=xA; this.yA=yA; this.xB=xB; this.yB=yB;
  }
  public Strich(int xA, int yA, int xB, int yB){
    this((float)xA, (float)yA, (float)xB, (float)yB);
  }
  public Strich(){this(0,0,0,0);}

  public Strich relatedTo(Strich s){
    Strich r=new Strich();
    float dx, dy;
    dx=s.xB-s.xA; dy=s.yB-s.yA; //vec(AB)
    float abSq=dx*dx+dy*dy;
    float dX, dY;
    dX=xA-s.xA; dY=yA-s.yA;     //vec(AA')
    r.xA=(dX*dx+dY*dy)/abSq;
    r.yA=(dY*dx-dX*dy)/abSq;
    dX=xB-s.xA; dY=yB-s.yA;     //vec(AB')
    r.xB=(dX*dx+dY*dy)/abSq;
    r.yB=(dY*dx-dX*dy)/abSq;
    return r;
  }

  // Strich zeichnet sich auf einer Graphik-Oberflche
  public void draw(Graphics g){
    g.drawLine((int)xA, (int)yA, (int)xB, (int)yB);
  }

  // hinterlt in Vector v Nachkommen nach Bauplan p fr den nchsten Durchgang
  public void nachkommen(Vector v, Vector p){
    float dx=xB-xA; float dy=yB-yA;
    Strich r;
    for (int i=p.size()-1;i>=0;i--){
      Strich n=new Strich();    //der neue absolute
      r=(Strich)p.elementAt(i); //der relative aus dem Plan
      n.xA=xA+r.xA*dx-r.yA*dy;
      n.yA=yA+r.xA*dy+r.yA*dx;
      n.xB=xA+r.xB*dx-r.yB*dy;
      n.yB=yA+r.xB*dy+r.yB*dx;
      v.addElement(n);
    }
  }
}

public class Fraktal extends Frame implements ActionListener{
  Vector bauplan=new Vector(5);
  Vector striche=new Vector(5);
  Strich basis;
  Fraktal.Canvas canvas;
  Button neub, nextb, printb;
  int gen;// Nummer der nchsten! Generation, wobei Basis: gen=0

  class Canvas extends java.awt.Canvas{
    public boolean mausAktiv=true;
    private int xA, yA, xB, yB;

    public Canvas(){
      super();
      addMouseListener(new MouseAdapter(){
        public void mousePressed(MouseEvent e){
          xA=e.getX(); yA=e.getY(); xB=xA; yB=yA;
        }
        public void mouseReleased(MouseEvent e){
          if(mausAktiv){
            xB=e.getX(); yB=e.getY();
            Strich s=new Strich(xA,yA,xB,yB);
            s.draw(getGraphics());
            if(basis==null) basis=s;
            else{
              striche.addElement(s);
              bauplan.addElement(s.relatedTo(basis));
              nextb.setEnabled(true);
            }
          }
        }
      });
      addMouseMotionListener(new MouseMotionAdapter(){
        public void mouseDragged(MouseEvent e){
          if (mausAktiv){
            Graphics g=getGraphics();
            g.setXORMode(Color.cyan);
            g.drawLine(xA,yA,xB,yB);
            xB=e.getX(); yB=e.getY();
            g.drawLine(xA,yA,xB,yB);
          }
        }
      });
    }

    public void update(Graphics g){
      if (striche==null) return;
      Strich s;
      for (int i=striche.size()-1;i>=0;i--){
        s=(Strich)striche.elementAt(i);  s.draw(g);
      }
      nextb.setEnabled(true);
    }

    public Dimension getMinimumSize(){
      return new Dimension(300,200);
    }
    public Dimension getPreferredSize(){
      return getMinimumSize();
    }
  }

  public Fraktal(){
    super("Besen-Fraktal");
    Panel p=new Panel();
    neub=new Button("neu");
    neub.addActionListener(this);
    p.add(neub);
    nextb=new Button("generiere 2");
    nextb.addActionListener(this);
    p.add(nextb);
    printb=new Button("print");
    printb.addActionListener(this);
    p.add(printb);
    add("North",p);
    canvas=new Fraktal.Canvas();
    add("Center",canvas);
    this.addWindowListener(new WindowAdapter(){
      public void windowClosing(WindowEvent e){System.exit(0);}
      });
    show();
    neu();
    pack();
  }

  public void actionPerformed(ActionEvent e){
    if (e.getSource()==neub) neu();
    else
    if (e.getSource()==nextb){
      canvas.mausAktiv=false;
      nextGeneration();
      canvas.repaint();
    }else
    if (e.getSource()==printb){
      PrintJob pj=getToolkit().getPrintJob(this,"Fraktal",null);
      if (pj!=null){
        Graphics g=pj.getGraphics();
        printTo(g);
        g.dispose();
        pj.end();
      }
    }
  }

  private void neu(){
    basis=null;
    bauplan.removeAllElements();
    striche.removeAllElements();
    clearCanvas();
    gen=2;
    nextb.setLabel("generiere 2");
    nextb.setEnabled(false);
    canvas.mausAktiv=true;
  }

  private Graphics clearCanvas(){
    Graphics g=canvas.getGraphics();
    Rectangle r=canvas.getBounds();
    g.clearRect(0,0,r.width,r.height);
    return g;
  }

  private void nextGeneration(){
    nextb.setEnabled(false);
    Vector v=new Vector(striche.size()*bauplan.size());
    Strich s;
    for (int i=striche.size()-1;i>=0;i--){
      s=(Strich)striche.elementAt(i);
      s.nachkommen(v,bauplan);
    }
    striche=v;
    nextb.setLabel("generiere "+ ++gen);
  }

  public void printTo(Graphics g){
    int n=gen-1;
    basis.draw(g);
    gen=1;
    striche.removeAllElements();
    striche.addElement(basis);
    for (;n>0;n--){
      nextGeneration();
      canvas.update(g);
    }
  }

  public static void main(String[] args){
    new Fraktal();
  }
}
