/* LinkedList --- doppelt verkettete Liste mit Cursor,
 * Interna (ListNode) nicht fr Clients sichtbar.
 */
import java.util.Enumeration;

class LinkedList implements Enumeration{
  private ListNode root;
  private ListNode cursor;

  public LinkedList(){
    root=new ListNode(); cursor=root;
  }

  //############# Setzen des Cursors und Operationen am Cursor #############
  public void first(){cursor=root.next;}

  public void last(){cursor=root.prev;}

  public void next(){
    if(cursor.next!=root) cursor=cursor.next;
  }

  public void prev(){
    if(cursor.prev!=root) cursor=cursor.prev;
  }

  public void setData(Object o){
    if (cursor!=root) cursor.data=o;
  }

  public Object getData(){
    if (cursor!=root) return cursor.data;
    else return null;
  }

  public void insert(Object data){//neues Element vor Cursor
    ListNode tmp=new ListNode(cursor.prev, cursor, data);
    cursor.prev.next=tmp;
    cursor.prev=tmp;
  }

  public void append(Object data){//neues Element hinter Cursor und diesen verschieben
    ListNode tmp=new ListNode(cursor, cursor.next, data);
    cursor.next.prev=tmp;
    cursor.next=tmp;
    this.next();
  }

  //lscht Listenelement an der Cursorposition
  public void delete(){
    if(cursor==root) return;
    cursor.prev.next=cursor.next;
    cursor.next.prev=cursor.prev;
    this.prev();
  }

  //sucht ab der Cursorposition nach einem Object.equal(o)
  public boolean search(Object o){
    ListNode n=cursor;
    while(n.next!=root){
      if(n.data.equals(o)){
        cursor=n;
        return true;
      }
      else n=n.next;
    }
    return false;
  }

  ListNode enumerationNode=null;

  public Enumeration elements(){
    enumerationNode=root;
    return this;
  }

  public boolean hasMoreElements(){
    return enumerationNode.next!=root;
  }

  public Object nextElement(){
    if(enumerationNode.next==root) return null;
    else{
      enumerationNode=enumerationNode.next;
      return enumerationNode.data;
    }
  }

  public String toString(){
    String s="Liste: ";
    ListNode n;
    for(n=root.next; n!=root; n=n.next) s=s+' '+n.data.toString();
    return s;
  }

  private Comparator comp=null;
  public void setComparator(Comparator c){comp=c;}

  /* merged i Teile von a und j Teile von b und hngt das Ergebnis an done an*/
  private ListNode mergeTo(ListNode done, ListNode a, int i, ListNode b, int j){
    while(i+j!=0){
      if(i!=0 && (j==0 || !comp.wantSwap(a.data, b.data))){//d.h. ...|| a.data<=b.data))
        done.prev=a;//Missbrauch von prev
        done=a;
        a=a.next;
        i--;
      }  else{
        done.prev=b;//Missbrauch von prev
        done=b;
        b=b.next;
        j--;
      }
    }
    return done;
  }

  /* Sortiert die Liste durch mehrfaches Mischen ihrer Teile.
     Da die Liste doppelt verkettet ist, kann die erste Verkettung - wie gewohnt -
     verwendet werden, um die Liste zu durchwandern; die zweite Verkettung wird missbraucht
     und dient whrend des Sortierens nicht fr Rckwrts-Verweise, sondern fr die
     aktualisierten Vorwrtsverweise. Nach jeder Durchwanderung wird das aber wieder korrigiert.
     */
  public void mergeSort(){
    ListNode done, todo, a, b;
    int n=1, i, j;
    do{
      done=root; todo=root.next;
      do{
        a=todo;
        b=a;    i=-1; while(++i<n &&    b!=root)    b=b.next;
        todo=b; j=-1; while(++j<n && todo!=root) todo=todo.next;
        done=mergeTo(done, a, i, b, j);
      }  while(todo!=root);
      done.prev=root;//Missbrauch von prev
      root.next=root.prev; done=root;
      do{todo=done.next; todo.next=todo.prev; todo.prev=done; done=todo;} while(todo!=root);
      n<<=1;
    }  while(a!=root.next);
    cursor=root;
  }

//############### Test der obigen Methoden #######################
  public static void main(String[] args){
    LinkedList ll=new LinkedList();
    int anz=Integer.parseInt(args[0]);
    for(int i=0; i<anz; i++) ll.append(new Integer((int)(Math.random()*1000)));
/*    ll.append(new Integer(10));
    ll.append(new Integer(7));
    ll.append(new Integer(17));
    ll.append(new Integer(32));
    ll.append(new Integer(10));
    ll.append(new Integer(17));
    ll.append(new Integer(2));
    ll.append(new Integer(91));
    ll.append(new Integer(12));
    ll.append(new Integer(12));
    ll.append(new Integer(7));
    ll.append(new Integer(10));
    */
    ll.setComparator(new Comparator(){
      public boolean wantSwap(Object a, Object b){
        return ((Integer)a).intValue()>((Integer)b).intValue();
      }
    });
    //System.out.println(ll);
    long t=System.currentTimeMillis();
    ll.mergeSort();
    System.out.println(System.currentTimeMillis()-t+" msec Sortierzeit fr "+anz+" Integers");
    //System.out.println(ll);

/*    ll.append("6");
    ll.append("2");
    ll.search("2");
    ll.append("3");
    ll.append("4");

    ll.first();
    ll.delete();

    ll.first();
    ll.insert("1");

    ll.last();
    ll.append("5");

    ll.first();
    ll.insert(new Double(0));

    Enumeration e; Object n;
    for(e=ll.elements(); e.hasMoreElements();){
      n=(Object)e.nextElement();
      System.out.println(n.toString());
    }
    System.out.println(ll.toString());

    LinkedList l=new LinkedList();
    long t1=System.currentTimeMillis();
    for(int i=0; i<10000; i++) l.append(new Integer(i));
    long t2=System.currentTimeMillis();
    System.out.println("Erzeugen von 10.000 Knoten in "+(t2-t1)+" msec.");

    Integer ii; boolean found;
    for(int j=0; j<100; j++){
      ii=new Integer((int)(10000*Math.random()));
      l.first();
      found=l.search(ii);
      if(!found) System.out.println("Error: node not found"+ii.toString());
      //else System.out.println("found "+ii.toString());
    }
    long t3=System.currentTimeMillis();
    System.out.println("Suchen von 100 zuflligen Knoten in "+(t3-t2)+" msec.");

    l.last();
    for(int i=0; i<9960; i++) l.delete();
    long t4=System.currentTimeMillis();
    System.out.println("Lschen von 9960 Knoten: in "+(t4-t3)+" msec.");

    System.out.println(l);
    */
  }
}
