/* Dies ist die Klasse auf dem "fremden Rechner". Die Methoden, die sie
   vom Interface Arbeiter implementiert, knnen von anderen Rechnern (die
   sich selber immer "lokale Rechner" nennen) aufgerufen werden.
*/

import java.rmi.*;
import java.rmi.server.*;
import java.rmi.registry.*;

public class ArbeitsSuchend extends UnicastRemoteObject implements Arbeiter{
  public ArbeitsSuchend() throws RemoteException{
    super();
  }

  public Object bearbeite(Job j){
    return j.losgehts();
  }

  /* Der Aufruf dieser Methode meldet diesen Rechner im Cluster als
     arbeitssuchend, d.h. ein einziger Arbeiter wird bei der lokalen
     RMIRegistry angemeldet (meist hat man auf dem Rechner ja auch nur
     einen Prozessor zur Verfgung. Falls mehrere Jobs vom gleichen
     Rechner gemacht werden sollen, werden automatisch mehrere Threads
     gestartet.)

     Aufrufbeispiel (alles in die gleiche Zeile):
       java -Djava.security.policy=Arbeiter.policy
            ArbeitsSuchend 1099
  */
  public static void main (String args[]){
    if (System.getSecurityManager()==null)
      System.setSecurityManager(new RMISecurityManager());
    int rmi=1099;//wenn ohne Argument, verwende Port 1099
    if(args.length>0) rmi=Integer.parseInt(args[0]);
    try{
      LocateRegistry.createRegistry(rmi);
      ArbeitsSuchend hier=new ArbeitsSuchend();
      Naming.rebind("rmi://localhost:"+rmi+"/EinArbeiter", hier);
      System.out.println("EinArbeiter steht auf Port "+rmi+" zur Verfgung.");
    } catch(Exception e){
      System.out.println(e.getMessage());
      e.printStackTrace();
    }
  }

  /* Testet, ob auf dem Rechner ip ein ArbeitsSuchender mit
     dem Namen EinArbeiter registriert ist, d.h. also, ob der betreffende
     (entfernte) Rechner bereit ist, Arbeit anzunehmen.
     ip ist eine IP-Adresse oder ein Rechnername optional gefolgt von
     einer :port-Angabe
     Diese Methode wird wahrscheinlich an verschiedenen Stellen gebraucht
     und deshalb hier zur Verfgung gestellt.
  */
  public static boolean isArbeitssuchend(String ip){
    Object obj;
    try{obj=Naming.lookup("rmi://"+ip+"/EinArbeiter");}
    catch(Exception ex){obj=null;}
    return obj!=null;
  }
}
