/* Ein Aufpasser wartet darauf, dass ein (entfernter) Arbeiter mit dem
   Job fertig wird. Dazu startet der Aufpasser einen neuen lokalen Thread.
   Am Ende des Threads bergibt er dem Interessenten das Ergebnis
   des Jobs. Wenn das Ergebnis niemanden interessiert, kann auch null
   als Interessent angegeben werden.
   Findet der Aufpasser den entfernten Arbeiter nicht, so wird kein Thread
   gestartet. Ob es geklappt hat, kann mit fortschritt() ermittelt werden.
*/

import java.rmi.*;
public class Aufpasser{
  public static final int UNERREICHBAR=0;
  public static final int ARBEITEND   =1;
  public static final int ERFOLGREICH =2;
  public static final int FEHLERHAFT  =-1;
  protected int fortschritt;

  public Aufpasser(final Job job, String rechner, final Interessent chef){
    final Arbeiter arbeiter;
    try{
      arbeiter=(Arbeiter)Naming.lookup("rmi://"+rechner+"/EinArbeiter");
      //Definition einer Anonymous Class, die Nachfahre von Thread ist
      Thread th=new Thread(){
        public void run(){
          try{
            Object obj=arbeiter.bearbeite(job);
            fortschritt=ERFOLGREICH;
            if(chef!=null) chef.nimmErgebnis(obj);
          } catch(Exception e){
            fortschritt=FEHLERHAFT;
            if(chef!=null) chef.nimmErgebnis(e);
          }
        }
      };
      //Der Thread muss noch gestartet werden. Das dauert praktisch keine Zeit.
      th.start();
      fortschritt=ARBEITEND;
    } catch(Exception ex){
      fortschritt=UNERREICHBAR;
    }
  }

  public int fortschritt(){return fortschritt;}

  //Aufpasser-static Interface, da sehr auf den Aufpasser bezogen;
  //vollstndiger Name: Aufpasser.Interessent
  public static interface Interessent{
    void nimmErgebnis(Object obj);
  }
}
