/* Gibt einem ArbeitsSuchenden Jobs zum Abarbeiten.

   Der Witz an der Sache ist, dass der ArbeitsSuchende schon lange laufen
   kann, wenn wir diese Klasse hier und die Jobs frisch erfinden. Wir
   mssen die neu entstehenden Dateien dem Arbeitssuchenden nicht einmal
   bringen. Alles ntige regelt RMI, was dadurch mglich wird, dass wir
   auf unserem Rechner einen Webserver laufen lassen. Dieser wird ebenfalls
   durch das vorliegende Programm gestartet.
   Der ArbeitsSuchende auf dem fremden Rechner steht uns also im vorliegenden
   Programm genauso zur Verfgung, als wenn er auf diesem Rechner hier laufen
   wrde.

   In diesem einfachen Beispiel warten wir noch auf das Ergebnis der Job-
   Bearbeitung. Dass der Job auf einem anderen Rechner bearbeitet wird,
   bringt uns also hier noch keine Zeitvorteile, ist aber gut frs Verstndnis.
*/
import java.rmi.*;
import java.math.BigInteger;
import java.util.Random;

public class PTesteZahl{
  public static void main(String args[]){
    if(args.length<1){
      System.out.println("Aufruf:\n"+
        "java -Djava.security.policy=Arbeiter.policy\n"+
        "     -Djava.rmi.server.codebase=http://localhost:2001/\n"+
        "     PTesteZahl ip_oder_name_des_beauftragten_Rechners");
      System.exit(1);
    }
    if (System.getSecurityManager()==null)
      System.setSecurityManager(new RMISecurityManager());
    try{
      Random zufall=new Random();
      //hole einen Arbeiter beim Rechner mit der angegebenen IP oder dem Namen
      Arbeiter arbeiter=(Arbeiter)Naming.lookup("rmi://"+args[0]+"/EinArbeiter");
      Job job;
      Object obj;

      //Erzeuge eine 250 Bit lange Zufallszahl, die 1-2^30 wahrscheinlich prim ist
      BigInteger bi=new BigInteger(250, 30, zufall);
      //generiere einen Primtester
      job=new JWahrscheinlichPrim(bi, 200);
      //lass den Arbeiter den Job bearbeiten und erhalte ein Ergebnis-Objekt
      obj=arbeiter.bearbeite(job);
      //Verarbeite das erhaltene Objekt geeignet.
      boolean prim=((Boolean)obj).booleanValue();
      String nicht=prim?"":" nicht ";
      System.out.println(bi+" ist"+nicht+" prim.");

      //Das ganze nochmal mit einem anderen Job
      long li=Math.abs(zufall.nextLong());
      job=new JPrim(li);          //neuer Job
      obj=arbeiter.bearbeite(job);//gleicher Arbeiter wie oben!
      //das erhaltene Objekt ist ein Boolean, von dem wir den Wert brauchen
      prim=((Boolean)obj).booleanValue();
      nicht=prim?"":" nicht ";
      System.out.println(li+" ist"+nicht+" prim.");
    } catch(Exception e){
      System.out.println (e.getMessage());
      e.printStackTrace ();
    }
  }
}
