import java.util.Random;

/**
 * Dies ist der abstrakte Vorfahr aller Sortieralgorithmen. Zu sortieren ist eine
 * Zahlenmenge in aufsteigender Reihenfolge. Diese wird von einem {@link Demonstrator}
 * mittels geeigneter Methoden zur Verfgung gestellt.
 */
public abstract class Algorithmus{
  private static Random zufall=new Random();
  /**
   * der Demonstrator, der die Zahlenmenge besitzt und Zugriff darauf bietet.
   */  
  protected Demonstrator demo;

  /**
   * erzeugt eine Menge von Zahlen, die einem Demonstrator zur Verwaltung gegeben
   * werden knnen. Sobald sie der Demonstrator hat, darf die Menge nicht mehr
   * verndert werden.
   * @param von kleinster gewnschter Zahlenwert (einschlielich).
   * @param bis grter gewnschter Zahlenwert (ausschlielich).
   * @param anz Gre der Zahlenmenge.
   * @param doppel true, wenn Doppelungen erlaubt sind, false, wenn nicht. Wenn
   * <CODE>anz &le bis-von</CODE>, sind Doppelungen automatisch erlaubt.
   * @return die erzeugte Zahlenmenge.
   */  
  public static int[] macheZahlen(int von, int bis, int anz, boolean doppel){
    int[] zahlen=new int[anz];
    int bereich=bis-von;
    if(anz>bereich) doppel=true;
    if(doppel) for(int i=0; i<anz; i++) zahlen[i]=von+zufall.nextInt(bereich);
    else for(int i=0; i<anz; i++) zahlen[i]=von+i;
    return zahlen;
  }
  
  /**
   * definiert, mit welchem Demonstrator die Sortierung durchgefhrt wird.
   * @param d ebendieser.
   */  
  public void setDemonstrator(Demonstrator d){
    demo=d;
  }
  
  /**
   * sortiert die Zahlenmenge des {@link Demonstrators} aufsteigend. Die Methoden
   * des Demonstrators sollen dabei fr den Zugriff und fr die geeignete Markierung
   * der Werte genutzt werden.
   */  
  public abstract void sortiere();
}
